import json
from typing import Dict, Optional

from case_prep import rower, updated_rows_with_decision_idx
from sqlite_utils import Database

from .prep import (
    decision_duplications,
    inserted_title_updater,
    setup_tables,
    single_idx,
    updated_cites,
    updated_containers,
    updated_titles,
    updated_units,
    upsert_statutes,
)
from .views import bulk_update_decision_scras


def insert_decision(x: Dict, db_path: str):
    """
    Implies pre-processed item from the acquisition library.
    """

    db = setup_tables(Database(db_path))  # create tables, if inexistent

    if not (item := decision_duplications(db, x)):
        return None  # only insert if no duplications; but possible row updates

    # item refers to a row for a Decision; add this to the table and retrieve pk
    idx = db["Decisions"].insert(item).last_pk

    # update identifiers (e.g. Dockets, Scras, etc.) via single_idx; then update Decision pk
    db["Decisions"].update(idx, single_idx(db, item))

    data = {}  # create prospective rows based on the inserted data
    if item.get("ponencia", None):
        # use decision pk as identifier of rows in other tables
        raw = rower(item["ponencia"], item.get("annex", None))
        data = updated_rows_with_decision_idx(idx, raw)

    # modify rows to get existing Citation pks; add rows to Citations
    # Citations are collected from every Decision's ponencia
    # Each Citation contains possible identifier: Docket, SCRA, Phil., Offg
    # If the identifier id doesn't exist, yet create it;
    # Otherwise fetch it prior Citation insertion
    # Post insertion, for Citations containing Docket and SCRA,
    # update the Decision's SCRA field
    if data.get("citations", None):
        rows = (updated_cites(db, x) for x in data["citations"])
        db["Citations"].insert_all(rows)
        db.executescript(bulk_update_decision_scras)  # update decision scras

    # modify rows to get existing Statute pks; add rows to Provisions
    if data.get("provisions", None):  # provisions contain statute data
        rows = upsert_statutes(db["CaseStatutes"], data["provisions"])
        db["Provisions"].insert_all(rows)

    # add rows to Segments and Annotations (with Segment pks)
    # each Segment is a line in a Decision
    # each Segment contains unlinked Citations
    # an Annotation is a linked Citation to a Segment
    # linking is done through identifier tables
    # at this juncture, some Segments do not have Citations since
    # some segments refer to cases like this X v. Y, supra.
    # so a Case Title is known but the Citation is not known
    if data.get("segments", None):
        for row in data["segments"]:
            annotations = row.pop("citations")  # remove first
            segment_id = db["Segments"].insert(row).last_pk
            if not annotations:
                continue
            add_data = {"segment_id": segment_id, "decision_id": idx}
            for anno in annotations:
                db["Annotations"].insert(single_idx(db, anno) | add_data)

    # with Segments & Citations added, use Segment pks / identifier pks
    # Case Titles are also collected from every Decision
    # Case Titles are associated with a corresponding Segment
    # Case Titles inserted can be updated further by matching cases like X v. Y, supra.
    # updated Case Titles can then be considered additional Annotations
    # use as test: 33678
    if data.get("titles", None):
        # prep insert
        rows = (updated_titles(db, x) for x in data["titles"])

        # insert proper
        db["Titles"].insert_all(rows)

        # create prospective Annotations
        db = inserted_title_updater(db, idx)

        # use same columns for Annotations and Titles
        cols = "segment_id, docket, phil, scra, offg"

        # previously annotated segments in a Decision
        aq = "decision_id = ?"
        anno_rows = db["Annotations"].rows_where(aq, (idx,), select=cols)

        # get updated title annotations in the same Decision
        tq = "decision_id = ? and updated = ?"
        title_rows = db["Titles"].rows_where(tq, (idx, 1), select=cols)
        add_data = {"decision_id": idx, "updated": 1}

        # create a new annotation, if not yet existing in the same Decision
        for t_row in title_rows:
            if t_row not in anno_rows:
                db["Annotations"].insert(t_row | add_data)

    return db


def insert_statute(item: Dict, db_path: str) -> Optional[Dict]:
    """
    Implies pre-processed item from the acquisition library.
    1. Ensure proper tables are setup
    2. Exit, if item is a duplicate
    3. Continue, insert to db, get pk of Statute row, if item is unique
    3. Add rows to Containers; pre-insert, update rows with idx
    4. Add rows to Units; pre-insert, update rows with idx

    """
    db = setup_tables(Database(db_path))  # create tables, if inexistent

    try:
        query = "category_num = ? and numeral = ?"
        args = (item["category_num"], item["numeral"])
        i = next(db["Statutes"].rows_where(query, args, select="id"))
        return None  # if can get i from item fields, statute already exists
    except StopIteration:
        pass

    u = item.pop("units")
    c = item.pop("containers")

    idx = db["Statutes"].insert(item).last_pk  # add to Statutes, retrieve pk

    if c and (containers := json.loads(c)):
        db["Containers"].insert_all(updated_containers(idx, containers))

    if u and (units := json.loads(u)):
        db["Units"].insert_all(updated_units(db["Containers"], idx, units))

    return db
