from sqlite_utils import Database


def identifiers(db: Database) -> Database:
    for table, column in [
        ("Dockets", "docket"),
        ("Phils", "phil"),
        ("Scras", "scra"),
        ("Offgs", "offg"),
    ]:
        if not db[table].exists():
            # needs to be a hash_id in tandem with get_or_create
            db[table].create({column: str}, hash_id="id")
            db[table].create_index(["id"])
            db[table].create_index([column])
    return db


def decisions(db: Database) -> Database:
    if not db["Decisions"].exists():
        db["Decisions"].create(
            columns={
                "origin": str,
                "date_prom": str,
                "docket": str,
                "phil": str,
                "scra": str,
                "offg": str,
                "case_title": str,
                "ponente": str,
                "annex": str,
                "category": str,
                "composition": str,
                "ruling_marker": str,
                "ponencia": str,
                "ruling": str,
                "fallo": str,
                "voting": str,
                "date_scraped": str,
                "date_updated": str,
                "orig_idx": str,
                "error": str,
                "initial": str,
            },
            pk="id",
            foreign_keys=["docket", "phil", "scra", "offg"],
        )
        for i in ["id", "date_prom", "docket", "scra", "phil", "offg"]:
            db["Decisions"].create_index([i])
    return db


def citations(db: Database) -> Database:
    if not db["Citations"].exists():
        db["Citations"].create(
            columns={
                "decision_id": int,
                "raw": str,
                "docket": str,
                "phil": str,
                "scra": str,
                "offg": str,
            },
            pk="id",
            column_order=(
                "id",
                "decision_id",
                "raw",
                "docket",
                "phil",
                "scra",
                "offg",
            ),
            foreign_keys=["decision_id", "docket", "phil", "scra", "offg"],
        )
        db["Citations"].create_index(["id"])
    return db


def segments(db: Database) -> Database:
    if not db["Segments"].exists():
        db["Segments"].create(
            columns={
                "segment_line": int,
                "decision_id": int,
                "content": str,
                "text": str,
                "keys": int,
                "footnotes": str,
            },
            pk="id",
            column_order=(
                "id",
                "decision_id",
                "segment_line",
                "content",
                "keys",
                "footnotes",
            ),
            foreign_keys=["decision_id"],
        )
        db["Segments"].create_index(["id"])
    return db


def annotations(db: Database) -> Database:
    if not db["Annotations"].exists():
        db["Annotations"].create(
            columns={
                "decision_id": int,
                "segment_id": int,
                "docket": str,
                "phil": str,
                "scra": str,
                "offg": str,
                "updated": bool,
            },
            pk="id",
            column_order=(
                "id",
                "segment_id",
                "docket",
                "phil",
                "scra",
                "offg",
            ),
            foreign_keys=[
                "decision_id",
                "segment_id",
                "docket",
                "phil",
                "scra",
                "offg",
            ],
            defaults={"updated": 0},
        )
        for i in [
            "segment_id",
            "docket",
            "scra",
            "phil",
            "offg",
        ]:
            db["Annotations"].create_index([i])
    return db


def titles(db: Database) -> Database:
    if not db["Titles"].exists():
        db["Titles"].create(
            columns={
                "decision_id": int,
                "segment_id": int,
                "raw": str,
                "key": int,
                "casename": str,
                "citation": str,
                "plaintiff": str,
                "defendant": str,
                "inre": str,
                "error": str,
                "updated": bool,
                "docket": str,
                "phil": str,
                "scra": str,
                "offg": str,
            },
            pk="id",
            foreign_keys=[
                "decision_id",
                "segment_id",
                "docket",
                "phil",
                "scra",
                "offg",
            ],
        )
        for i in [
            "segment_id",
            "docket",
            "scra",
            "phil",
            "offg",
        ]:
            db["Titles"].create_index([i])
    return db


def case_statutes(db: Database) -> Database:
    if not db["CaseStatutes"].exists():
        db["CaseStatutes"].create(
            {
                "category": str,
                "numeral": str,
            },
            pk="id",
        )
    return db


def provisions(db: Database) -> Database:
    if not db["Provisions"].exists():
        db["Provisions"].create(
            {
                "text": str,
                "casestatute_id": int,
                "provision": str,
                "decision_id": str,
            },
            pk="id",
            foreign_keys=["casestatute_id", "decision_id"],
        )
    return db
