from typing import Dict, Optional

from sqlite_utils import Database


def get_create(
    db: Database, column: str, table: str, idx: str
) -> Optional[str]:
    """
    Get an identifier pk, if existing, from any of the following tables:
    Dockets, Phils, Scras, Offgs; if inexistent, insert record.
    """
    if not idx:
        return None
    try:
        query = f"{column} = ?"
        args = (idx,)
        return next(db[table].rows_where(query, args, select="id"))["id"]
    except StopIteration:
        return db[table].insert({column: idx}, hash_id="id").last_pk


def single_idx(db: Database, x: Dict):
    """ Get or create from the Dockets, Phils, Scras, Offgs tables based on record x"""
    return {
        "docket": get_create(db, "docket", "Dockets", x.get("docket", None)),
        "phil": get_create(db, "phil", "Phils", x.get("phil", None)),
        "scra": get_create(db, "scra", "Scras", x.get("scra", None)),
        "offg": get_create(db, "offg", "Offgs", x.get("offg", None)),
    }


def updated_cites(db: Database, record: Dict) -> Dict:
    """
    1. Part of for loop with multiple raw Citation rows.
    2. For each record, get pk from applicable Dockets, Phils, Scras, Offgs tables.
    3. Adds other fields to updated dict.
    """
    ids = single_idx(db, record)
    ids.update({"raw": record["raw"], "decision_id": record["decision_id"]})
    return ids


def updated_titles(db: Database, record: Dict) -> Dict:
    """
    1. Part of for loop with multiple raw Title rows.
    2. Assumes that segments have already been added
    3. For each record, get pk from applicable Dockets, Phils, Scras, Offgs tables.
    4. Adds other fields to updated dict.
    """
    ids = single_idx(db, record)
    query = "decision_id = ? and segment_line = ?"
    args = (record["decision_id"], record["segment_line"])
    seg_id = next(db["Segments"].rows_where(query, args, select="id"))["id"]
    ids.update(
        {
            "segment_id": seg_id,
            "decision_id": record["decision_id"],
            "raw": record["raw"],
            "key": record["key"],
            "casename": record["casename"],
            "citation": record["citation"],
            "plaintiff": record["plaintiff"],
            "defendant": record["defendant"],
            "inre": record["inre"],
            "error": record["error"],
            "updated": record["updated"],
        }
    )
    return ids
