from typing import Dict, Optional

from sqlite_utils import Database

from casedb.sql.prevent_dkt_dups import input_docket_sql
from casedb.sql.prevent_phil_dups import input_phil_sql, null_decision_phil


def decision_duplications(db: Database, x: Dict) -> Optional[Dict]:
    """ If item is returned, then no duplicates were found in processing."""
    if not (item := dup_phil_entry(db, x)):
        return None
    if not (item := dup_docket_entry(db, x)):
        return None
    return item


def dup_phil_entry(db: Database, item: Dict) -> Optional[Dict]:
    # if key inexistent in item, no need to determine duplication
    if not item.get("phil", None):
        return item

    # is the phil existing? if yes, get existing docket of existing phil
    result = db.execute(input_phil_sql, (item["phil"],)).fetchone()

    # no matching phil record, return item for insertion
    if not result:
        return item

    # determine if existing docket matches the prospective docket
    if item["docket"] == result[0]:  # 0 means first column of tuple
        return None  # marker to disallow entry of the item

    # since existing docket != prospective docket, make null decision phil
    db.execute(null_decision_phil, (item["phil"],))  # prevents duplicates
    item["phil"] = None  # can insert the item, but phil needs to be None
    return item  # return item for insertion


def dup_docket_entry(db: Database, item: Dict) -> Optional[Dict]:
    # if key inexistent in item, no need to determine duplication
    if not item.get("docket", None):
        return None

    # is the docket existing? if yes, get existing docket
    result = db.execute(input_docket_sql, (item["docket"],)).fetchone()

    # no matching docket, return item for insertion
    if not result:
        return item

    if (  # special rule: if no existing Phil in db, add phil; update decision phil
        result  # means an existing docket is present, i.e. decision.docket
        and result[1]  # decision pk is included in the results
        and not result[2]  # third column of tuple ("decision.phil") is empty
        and item.get("phil", None)  # valid value exists for decision.phil
        and not list(db["Phils"].rows_where("phil = ?", (item["phil"],)))
    ):  #
        pk = db["Phils"].insert({"phil": item["phil"]}).last_pk
        db["Decisions"].update(result[1], {"phil": pk})  # updates decision pk

    # only when no result, is item allowed; but see updating rule
    return None
