from typing import Dict, List, Optional

from sqlite_utils.db import Table


def updated_containers(idx: int, containers: List[Dict]) -> List[Dict]:
    """
    Since a statute was recently inserted, update the prospective container row with the statute id
    """
    for container in containers:
        container["statute_id"] = idx
    return containers


def updated_units(tbl: Table, idx: int, units: List[Dict]) -> List[Dict]:
    """
    Since a statute and a possible container were recently inserted, update the prospective unit row with the statute id and the potential container id.
    """
    for unit in units:
        unit["statute_id"] = idx
        try:
            query = "statute_id = ? and container_id = ?"
            args = (idx, unit["container"])
            row = next(tbl.rows_where(query, args, select="id"))
            unit["container"] = row["id"]
        except StopIteration:
            unit["container"] = None
    return units


def get_create_statute(tbl: Table, cat: str, num: str) -> Optional[int]:
    """Get CaseStatutes pk, if existing; if inexistent, insert the record."""
    if not cat or not num:
        return None
    try:
        query = "category = ? and numeral = ?"
        args = (cat, num)
        return next(tbl.rows_where(query, args, select="id"))["id"]
    except StopIteration:
        return tbl.insert({"category": cat, "numeral": num}).last_pk


def upsert_statutes(tbl: Table, rows: List[Dict]):
    """
    For each row, remove the title and num keys; use as parameters to "get or create" a CaseStatutes pk and populate the field.
    1. This function updates the CaseStatutes table
    2. This function makes the rows "insertable" to the Provisions table
    """
    for row in rows:
        category = row.pop("title")
        numeral = row.pop("num")
        row["casestatute_id"] = get_create_statute(tbl, category, numeral)
    return rows
